// receive POCSAG-message from serial port
// transmit message on 433.900 Mhz using si4432-based "RF22" ISM modules

// This is the "non-ham" version of the application. All references
// to ham call-signs and the CW beacon has been removed

// this program uses the arduino RadioHead library
// http://www.airspayce.com/mikem/arduino/RadioHead/

// (c) 2014 Kristoff Bonne (ON1ARF)

// Version 0.1.0 (20140330) Initial release
// Version 0.1.1 (20140418) pocsag message creation now done on arduino
// Version 0.1.2 (20140516) using "Pocsag" class, moved to ReadHead library, removed callsign and CW beacon

#include <SPI.h>
#include <RH_RF22.h>

// pocsag message generation library
#include <Pocsag.h>

// Singleton instance of the radio
RH_RF22 rf22;

Pocsag pocsag;

void setup() 
{  
  Serial.begin(9600);
  if (rf22.init()) {
    Serial.println("RF22 init OK");
  } else {
    Serial.println("RF22 init FAILED");
  }; // end if

  // Set frequency. AfcPullinRange set to 50 Khz (unused, we are not receiving)
  //  rf22.setFrequency(433.9000, 0.00);
  rf22.setFrequency(439.9875, 0.050); // subtract 30 Khz for crystal correction
                                          // this correction-value will differ per device.
                                          // to be determined experimental

  // set to 512 bps FSK, 4.5 Khz deviation (POCSAG 1200)
  rf22.setModemConfig(RH_RF22::FSK_Rb_1200Fd4_5 ); 

  // 6 mW TX power (EU ISM legislation)
  rf22.setTxPower(RH_RF22_TXPOW_20DBM);
  
}

void loop() {
//vars
int rc;
int state; // state machine for CLI input

// bell = ascii 0x07
char bell=0x07;

// data
long int address;
int addresssource = 1;
int repeat = 1;
char textmsg[42]; // to store text message;
int msgsize;

// read input:
// format: "<address> <message>"

Serial.println("Format:");
Serial.println("<address> <message>");

// init var
state=0;
address=0;
addresssource=1;
msgsize=0;

while (state >= 0) {
char c;
  // loop until we get a character from serial input
  while (!Serial.available()) {
  }; // end while

  c=Serial.read();


  // break out on ESC
  if (c == 0x1b) {
    state=-999;
    break;
  }; // end if

  // state 1: address ("0" to "9")
  if (state == 0) {
    if ((c >= '0') && (c <= '9')) {
      long int newaddress;

      newaddress = address * 10 + (c - '0');

      if (newaddress <= 0x1FFFFF) {
        // valid address
        address=newaddress;

        Serial.write(c);
      } else {
        // address to high. Send "beep"
        Serial.write(bell);
      }; // end else - if

    } else if (c == ' ') {
      // received space, go to next field (address source)
      Serial.write(c);
      state=2;
    } else {
      // error: echo "bell"
      Serial.write(bell);
    }; // end else - elsif - if

    // get next char
    continue;
  }; // end state 2

  // state 2: message, up to 40 chars, terminate with cr (0x0d) or lf (0x0a)
  if (state == 2) {
    // accepted is everything between space (ascii 0x20) and ~ (ascii 0x7e)
    if ((c >= 0x20) && (c <= 0x7e)) {
      // accept up to 40 chars
      if (msgsize < 40) {
        Serial.write(c);

        textmsg[msgsize]=c;
        msgsize++;
      } else {
        // to long
        Serial.write(bell);
      }; // end else - if
                        
    } else if ((c == 0x0a) || (c == 0x0d)) {
      // done
   
      Serial.println("");
                        
      // add terminating NULL
      textmsg[msgsize]=0x00;

      // break out of loop
      state=-1;
      break;

    } else {
      // invalid char
      Serial.write(bell);
    }; // end else - elsif - if

    // get next char
    continue;
  }; // end state 8;

}; // end while


// Function "P": Send PAGE
if (state == -1) {
  Serial.print("address: ");
  Serial.println(address);

  Serial.print("addresssource: ");
  Serial.println(addresssource);

  Serial.print("repeat: ");
  Serial.println(repeat);

  Serial.print("message: ");
  Serial.println(textmsg);
  
  // create pocsag message
  // batch2 option = 0 (truncate message)
  // invert option1 (invert)

  rc=pocsag.CreatePocsag(address,addresssource,textmsg,0,1);

  if (!rc) {
    Serial.print("Error in createpocsag! Error: ");
    Serial.println(pocsag.GetError());

    // sleep 10 seconds
    delay(5000);
  } 

  else {

   
    // send at least once + repeat
    for (int l=-1; l < repeat; l++) {
      Serial.println("POCSAG SEND");

      rf22.send((uint8_t *)pocsag.GetMsgPointer(), pocsag.GetSize());
      rf22.waitPacketSent();
      
      delay(3000);
    }; // end for

  }; // end else - if;
  
};
}; // end main application
